/*
 *
 * OpenSBC.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (c) opensipstack.org.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is OpenSIPStack.
 *
 * The Initial Developer of the Original Code is opensipstack.org.
 *
 * The author of this code is Joegen E. Baclor
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: OpenSBC.h,v $
 * Revision 1.26  2007/07/15 12:27:26  joegenbaclor
 * Added capability to route to registered endpoints using static routes
 * Removed unused config parameters
 *
 * Revision 1.25  2007/07/15 07:51:20  joegenbaclor
 * implemented resource counter page
 *
 * Revision 1.24  2007/07/14 07:53:56  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.23  2007/07/06 14:18:25  rcolobong
 * Add Upstream Proxy Route feature
 *
 * Revision 1.22  2007/06/13 09:22:01  joegenbaclor
 * Added IVR handler
 *
 * Revision 1.21  2007/05/18 01:23:55  joegenbaclor
 * 1.  Added separate useragent to handle B2BUpperReg backdoor
 * 2.  Modified routing for B2BUpperReg
 * 3.  Corrected bug in ParserTools::GetCompactHeader()
 *
 * Revision 1.20  2007/05/07 06:13:22  rcolobong
 * 1. Add support for disabling and enabling Local Refer
 * 2. Add support for defining static RTP media address
 * 3. Remove support for Force B2B Routes
 *
 * Revision 1.19  2007/03/07 23:20:26  joegenbaclor
 * Added media server UA
 *
 * Revision 1.18  2007/02/16 11:09:20  joegenbaclor
 * More work on privacy
 *
 * Revision 1.17  2007/02/15 09:36:29  joegenbaclor
 * Added Privacy Section
 *
 * Revision 1.16  2007/02/14 11:22:09  joegenbaclor
 * Added Trusted Domain section
 *
 * Revision 1.15  2007/01/17 10:33:43  joegenbaclor
 * Redesigned syslog
 *
 * Revision 1.14  2007/01/17 00:05:59  joegenbaclor
 * Added SysLog
 *
 * Revision 1.13  2007/01/12 10:50:05  joegenbaclor
 * minor proxy bug fixes
 * more launcher code
 *
 * Revision 1.12  2007/01/12 01:02:50  joegenbaclor
 * launcher related code
 *
 * Revision 1.11  2007/01/10 23:58:30  joegenbaclor
 * launcher specific code
 *
 * Revision 1.10  2007/01/08 11:15:18  joegenbaclor
 * Added registration status page
 *
 * Revision 1.9  2007/01/08 07:13:23  joegenbaclor
 * Added ability to run SBC in pure proxy or pure b2b modes
 *
 * Revision 1.8  2006/12/22 07:40:27  joegenbaclor
 * More on command line argument parsing.
 *
 * Revision 1.7  2006/11/22 11:37:09  rcolobong
 * 1. Added new support for Force B2BUA in HTTP configuration
 * 2. Added log level in Application level and SIP/RTP Level
 * 3. Update OnAcceptRegistration signature
 *
 * Revision 1.6  2006/10/11 04:59:50  rcolobong
 * Add http configuration "Accept all registration". If this true then it will accept all registration even if it is not included on the "Local domain accounts"
 *
 * Revision 1.5  2006/09/01 12:51:03  rcolobong
 * Use ActiveSessionCounter to update session count through OSSApplication
 *
 * Revision 1.4  2006/08/30 09:07:31  rcolobong
 * Update setting connection timer and alerting timer
 *
 * Revision 1.3  2006/08/29 07:43:58  rcolobong
 * 1. Rename file to a better filename
 * 2. Change SBCRTBESesionManager class to RTBESessionManager
 *
 * Revision 1.2  2006/08/16 10:34:32  rcolobong
 * 1. Syslog server is now configurable
 * 2. Alerting timer and Connection Timer is now configurable
 *
 * Revision 1.1  2006/08/14 10:04:59  rcolobong
 * Convert B2BUA to SBC
 * Support MP logging
 *
 * Revision 1.5  2006/07/11 13:59:34  joegenbaclor
 * Completed Proxy functionalities for B2BUA
 *
 * Revision 1.4  2006/07/10 06:29:37  joegenbaclor
 * Completed crude Registration support for B2BUA
 *
 * Revision 1.3  2006/07/03 15:29:47  joegenbaclor
 * Started merging OpenB2BUA into OpenSBC
 *
 * Revision 1.2  2006/06/20 09:58:11  joegenbaclor
 * *** empty log message ***
 *
 * Revision 1.1  2006/06/14 08:43:38  joegenbaclor
 * Initial upload of OpenB2BUA applcation and related classes
 *
 *
 */

#ifndef OPENSBC_H
#define OPENSBC_H

#include <ptlib.h>
#include "OSSApplication.h"
#include "B2BUA.h"
#include "MediaServer.h"

using namespace B2BUA;

class OpenSBCDaemon;
class SBCRoutingHandler;
class SBCCallHandler;
class SBCAuthHandler;
class SBCIVRHandler;

class OpenSBC : public B2BUserAgent
{
  PCLASSINFO( OpenSBC, B2BUserAgent );
public:


  // internal OpenSIPStack headers
  class UpstreamProxyRoute : public SIPHeaderF 
  { 
  PCLASSINFO( UpstreamProxyRoute, SIPHeaderF ); 
  public: 
    UpstreamProxyRoute(); 
    UpstreamProxyRoute(const SIPHeaderF & header); 
    UpstreamProxyRoute(const OString & header); 
    UpstreamProxyRoute(const MimeHeader & header); 
    UpstreamProxyRoute(const OStringArray & header); 
    UpstreamProxyRoute(const RouteURI::Collection & uri); 
  }; 

  class UpstreamProxyRouteList : public SIPHeaderFList 
  { 
  PCLASSINFO( UpstreamProxyRouteList, SIPHeaderFList ); 
  public: 
    UpstreamProxyRouteList(); 
    UpstreamProxyRouteList(const SIPHeaderFList & list); 
    UpstreamProxyRouteList(const SIPHeaderF::Collection & list); 
  };

  OpenSBC( 
    OpenSBCDaemon * application,
    UAMode mode
  );

  virtual void OnStart( 
    OSSAppConfig & config 
  );

  virtual void OnStop();

  virtual void OnConfigChanged( 
    OSSAppConfig & config 
  );

  virtual B2BCallInterface * OnCreateCallInterface();

  virtual B2BAuthInterface * OnCreateAuthInterface();

  virtual B2BMediaInterface * OnCreateMediaInterface();

  virtual B2BRoutingInterface * OnCreateRoutingInterface();

  virtual B2BIVRInterface * OnCreateIVRInterface();

  virtual RegisterSession::AcceptRegistrationResponse OnAcceptRegistration( 
    RegisterSession & session,
	  const SIPMessage & request
  );

  virtual BOOL OnIncomingCall(
    B2BUAConnection & connection,
    B2BUACall & call,
    SIPMessage & invite
  );

  BOOL FindLocalDomainAccount(
    const SIPURI & uri,
    SIPURI & accountURI
  )const;

  BOOL AddLocalDomainAccounts(
    const PStringArray & accounts,
    BOOL dumpExisting
  );

  BOOL AddTrustedDomains(
    const PStringArray & accounts,
    BOOL dumpExisting
  );

  BOOL IsTrustedDomain( const SIPURI & uri )const;

  BOOL AddPrivacyTrustedDomains(
    const PStringArray & accounts,
    BOOL dumpExisting
  );

  BOOL IsPrivacyTrustedDomain( 
    const SIPURI & uri,
    SIPURI & domainEntry
  )const;

  PINLINE const PString & GetPrivacyDefaultRealm()const{ return m_PrivacyDefaultRealm; };
  PINLINE void SetPrivacyDefaultRealm( const PString & realm ){ m_PrivacyDefaultRealm = realm; };
  PINLINE BOOL WillAcceptAllRegistration()const{ return m_AcceptAllRegistration; };
  PINLINE BOOL WillAcceptAllCalls()const{ return m_AcceptAllCalls; };
  PINLINE OpenSBCDaemon * GetApplication(){ return m_Application; };
  PINLINE SBCAuthHandler * GetSBCAuthHandler(){ return m_SBCAuthHandler; };
  PINLINE SBCCallHandler * GetSBCCallHandler(){ return m_SBCCallHandler; };
  PINLINE SBCRoutingHandler * GetSBCRoutingHandler(){ return m_SBCRoutingHandler; };

protected:
  OpenSBCDaemon * m_Application;

  BOOL m_AcceptAllRegistration;
  BOOL m_AcceptAllCalls;

  SBCAuthHandler * m_SBCAuthHandler;
  SBCCallHandler * m_SBCCallHandler;
  SBCRoutingHandler * m_SBCRoutingHandler;
  SBCIVRHandler * m_SBCIVRHandler;

  PMutex m_LocalDomainAccountsMutex;
  PDICTIONARY( AccountList, PCaselessString, SIPURI );
  AccountList m_LocalDomainAccounts;

  PMutex m_TrustedDomainsMutex;
  PSortedStringList m_TrustedDomains;

  PMutex m_PrivacyTrustedDomainsMutex;
  PSortedStringList m_PrivacyTrustedDomains;
  PString m_PrivacyDefaultRealm;
};

//////////////////////////////////////////////////////////////////

class OpenSBCDaemon : public OSSApplication
{
  PCLASSINFO( OpenSBCDaemon, OSSApplication );
public:

  OpenSBCDaemon();

  ~OpenSBCDaemon();

  
  // pure virtual from OSSApplication
  virtual void OnStart( 
    OSSAppConfig & config 
  );

  virtual void OnStop();

  virtual void OnConfigChanged( 
    OSSAppConfig & config 
  );

  PDECLARE_NOTIFIER( PThread, OpenSBCDaemon, OnConfigChanged );

  virtual void OnHandleCommandArgs(
      PINDEX i,
      const PString & arg
  );

  virtual OSSAppConfig * OnCreateAppConfig();

  virtual void OnAdditionalHTTPResource( 
    PHTTPSimpleAuth & authority
  );

  virtual void OnAdditionalHTTPLinks( 
    PHTML & html 
  );

  PString OnLoadRegistrationStatus( 
    const PString & htmlBlock 
  );
  
  PString OnLoadResourceCounters( 
    const PString & htmlBlock 
  );

  BOOL OnPostControl(
    const PStringToString & data, 
    PHTML & msg
  );

  virtual void OnAdditionalXMLRPCMethods(
    PStringArray & moreMethods
  );

  virtual void OnCustomXMLRPCRequest(
    const PString & method,
    PArray<PStringToString> & request,
    PArray<PStringToString> & response
  );

  virtual const PString& GetName() const;

  void SetPTraceLevel( unsigned level );

  PINLINE OpenSBC * GetSBC(){ return m_SBC; };
  PINLINE const PString & GetInstanceId()const{ return m_InstanceId; };
  PINLINE const SIPURI & GetExtendedArgs()const{ return m_ExtendedArgs; };
  PINLINE BOOL HasExtendedArgs()const{ return m_HasExtendedArgs; };
protected:
  OpenSBC * m_SBC;
  B2BUAEndPoint * m_EndPoint;
  B2BUAEndPoint * m_BackDoorEndPoint;
  B2BUserAgent::Registrar * m_Registrar;
  B2BUserAgent::Proxy * m_Proxy;
  MS::MediaServer * m_MediaServer;
  PString m_InstanceId;
  PString m_ExtendedAppName;
  SIPURI m_ExtendedArgs;
  BOOL m_HasExtendedArgs;
  BOOL m_IsInitialized;
};

class SBCRegistrationStatusPage : public PServiceHTTPString
{
  PCLASSINFO( SBCRegistrationStatusPage, PServiceHTTPString );

  public:
    SBCRegistrationStatusPage(
      OpenSBCDaemon & app, PHTTPAuthority & auth);
    
    virtual BOOL Post(
      PHTTPRequest & request,
      const PStringToString &,
      PHTML & msg
    );
  
  private:
    OpenSBCDaemon & app;
};

class SBCResourceCounterPage : public PServiceHTTPString
{
  PCLASSINFO( SBCResourceCounterPage, PServiceHTTPString );

  public:
    SBCResourceCounterPage(
      OpenSBCDaemon & app, PHTTPAuthority & auth
    );
      
  private:
    OpenSBCDaemon & app;
};

#endif




