/*
 *
 * Router.cxx
 *
 * Open SIP Stack ( OSS )
 *
 * Copyright (c) opensipstack.org.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is OpenSIPStack.
 *
 * The Initial Developer of the Original Code is opensipstack.org.
 *
 * The author of this code is Joegen E. Baclor
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * The OpenSIPStack Library includes some GPL/LGPL code libraries that MAY be 
 * enabled at compile time using the --enable-gpllibs configure switch.  If
 * enabled, the content of this file is published under the terms of GNU 
 * General Public License Version 2.  For a detailed list of the available
 * GPL code, see $(opensipstack)/gnu/README file.
 *
 * $Log: Router.cxx,v $
 * Revision 1.17  2007/07/15 12:27:26  joegenbaclor
 * Added capability to route to registered endpoints using static routes
 * Removed unused config parameters
 *
 * Revision 1.16  2007/07/14 07:53:56  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.15  2007/07/07 15:21:17  joegenbaclor
 * Added iproute2 support for linux
 *
 * Revision 1.14  2007/07/06 14:16:36  rcolobong
 * 1. Route by primary codec
 * 2. Support Upstream proxy routing
 *
 * Revision 1.13  2007/06/18 15:30:36  joegenbaclor
 * Implemented Internal DNS Mapping
 *
 * Revision 1.12  2007/06/17 02:49:25  joegenbaclor
 * Introduced Global lock to PHTTPConfig
 *
 * Revision 1.11  2007/06/13 09:22:01  joegenbaclor
 * Added IVR handler
 *
 * Revision 1.10  2007/05/29 17:43:17  joegenbaclor
 * Initial work on auto attendant
 *
 * Revision 1.9  2007/05/14 06:24:31  joegenbaclor
 * Added DNS Failover and ICMP validation
 *
 * Revision 1.8  2007/04/03 06:06:48  rcolobong
 * 1. Add new method GetAllTargetURIList
 * 2. Update handling of MaxSession in SIPURI parameter
 *
 * Revision 1.7  2007/04/03 04:38:12  joegenbaclor
 * Made sure CALL_TRANSFER_PREFIX is stripped when routing through local registration database
 *
 * Revision 1.6  2007/01/22 10:00:58  joegenbaclor
 * Fixed ProxyRouteRequest
 *
 * Revision 1.5  2007/01/17 00:08:44  joegenbaclor
 * Added SysLog
 *
 * Revision 1.4  2007/01/08 07:13:23  joegenbaclor
 * Added ability to run SBC in pure proxy or pure b2b modes
 *
 * Revision 1.3  2006/11/22 11:33:25  rcolobong
 * 1. Change method FindRoute to HasScheme
 * 2. Fix problem where it "Rounds Robin" in HasScheme method
 *
 * Revision 1.2  2006/10/11 05:04:50  rcolobong
 * Add new method FindRoute where it will search in "OpenSBC Route" based on scheme
 *
 * Revision 1.1  2006/08/14 10:04:59  rcolobong
 * Convert B2BUA to SBC
 * Support MP logging
 *
 * Revision 1.5  2006/08/04 05:14:03  rcolobong
 * 1.  Add getter for Routes
 * 2.  Add support for RTBE Scheme in FindRoute
 *
 * Revision 1.4  2006/07/17 11:36:44  joegenbaclor
 * 1.  More routing enhancements to B2BUA
 *
 * Revision 1.3  2006/07/13 06:53:03  joegenbaclor
 * 1.  Introduced Sanity checks for incoming SIP Message
 * 2.  Corrected bug in SIPURI where enum scheme is not getting recognized.
 * 3.  Added ENUM support to routing
 * 4.  Introduced global routing flag to indicate when and not to rewrite outbound To URI's
 *
 * Revision 1.2  2006/06/23 10:30:54  joegenbaclor
 * This revision marks the first successful call transfer via REFER by OpenB2BUA
 *
 * Revision 1.1  2006/06/20 09:58:11  joegenbaclor
 * *** empty log message ***
 *
 * Revision 1.2  2006/05/19 06:30:37  joegenbaclor
 * 1.  Fixed bug in SIPHeaderB where tag may not be parsed properly if no < > enclosure is present
 * 2.  Various enhancements to OpenSBC and proxy session
 *
 * Revision 1.1  2006/05/17 04:04:48  joegenbaclor
 * Initial upload of OpenSBC files
 *
 *
 */

#include "Router.h"
#include "SDPLazyParser.h"

using namespace SDP;

#define new PNEW

Router::Router()
{
  m_ICMPValidate = FALSE;
  m_MaskTargetURI = FALSE;
}

Router::Router( 
  const OStringArray & routes
)
{
  m_ICMPValidate = FALSE;
  m_MaskTargetURI = FALSE;
  Parse( routes );
}

Router & Router::operator=( 
  const OStringArray & routes
)
{
  Parse( routes );
  return *this;
}

BOOL Router::Parse( 
  const OStringArray & routes
)
{
  PWaitAndSignal lock( m_RoutesMutex );

  if( routes.GetSize() == 0 )
    return FALSE;

  m_Routes.RemoveAll();

  for( PINDEX i = 0; i < routes.GetSize(); i++ )
    AppendRoute( routes[i] );

  return m_Routes.GetSize() != 0;
}

BOOL Router::AppendRoute(
  const OString & route
)
{
  PWaitAndSignal lock( m_RoutesMutex );

  RouteRecord * rec = new RouteRecord( this );
  if( !rec->Parse( route ) )
    delete rec;
  else
  {
    PTRACE( 1, "Appending route: " << *rec );
    m_Routes.Append( rec );
    return TRUE;
  }

  return FALSE;
}

BOOL Router::FindRoute(
  const SIPMessage & request,
  SIPURI & target,
  BOOL incrementCurrentRoute,
  BOOL compareMethods
)
{
  PWaitAndSignal lock( m_RoutesMutex );

  if( !request.IsRequest() )
    return FALSE;

  /// get the To URI
  SIPURI to = request.GetToURI();

  OString userName = to.GetUser();

  BOOL ok = FALSE;
  for( PINDEX i = 0; i < m_Routes.GetSize(); i++ )
  {
    ok = FALSE;
    if(  m_Routes[i] *= to )
    {
      if( incrementCurrentRoute )
      {
        ok = m_Routes[i].GetTargetURI( target );
      }else
      {
        ok = m_Routes[i].GetTargetURI( target, 0 );
      }

      if( ok )
      {
        if( compareMethods )
        {
          /// check if the target uri has a method specified
          OString method;
          if( target.GetParameter( "method", method ) )
          {
            method = ParserTools::UnQuote( method );
            LOG( LogDebug(), "method=" << method << "request.method=" << request.GetMethod() );
            if( request.GetMethod() *= method )
              break;
            else
              continue;
          }else
          {
            LOG( LogDebug(), "method=?" << "request.method=" << request.GetMethod() );
            if( request.IsInvite() )
              break;
            else
              continue;  //searching
          }
        }

        break;
      }
    }
  }

  
  if( ok )
  {
    PIPSocket::Address addr;
    WORD port = 0;

    if( !SIPTransport::Resolve( target, addr, port, SIPTransport::UDP, m_ICMPValidate ) )
      return FALSE;

    if( target.GetUser().IsEmpty() )
      target.SetUser( to.GetUser() );

    PTRACE( 1, "-->> From: " << request.GetFromURI() 
    << " Target: " << request.GetMethod() << " " << target );
  }else
  {
    PTRACE( 1, "*** NO STATIC ROUTE DEFINED *** From: " << request.GetFromURI()  
      <<  " Target: " <<  to );
  }

  return ok;
}


BOOL Router::FindRoute(
  SIPTransportManager & transportManager,
  RegistrationDatabase & regDb,
  const SIPMessage & invite,
  B2BUAConnection & connection,
  BOOL useFromURI
)
{
  PWaitAndSignal lock( m_RoutesMutex );

  /// get the To URI
  SIPURI to = invite.GetToURI();
  OString targetName = to.GetUser();

  /// get the From URI
  SIPURI from = invite.GetFromURI();
  OString userName = from.GetUser();

  // get primary media codec here
  SDPLazyParser sdpParser = invite.GetBody();

  // value required for media format
  int payloadType = 0;
  OString formatName;
  SIP::SIPURI mediaURI;
  OString primaryCodec;

  // Get Media Offset
  if( sdpParser.GetMediaFormat( SDPLazyParser::Audio, 0, 0,
    payloadType, formatName, mediaURI ) )
  {
    primaryCodec = formatName;
  }

  BOOL ok = FALSE;
  for( PINDEX i = 0; i < m_Routes.GetSize(); i++ )
  {
    if( ( useFromURI && (m_Routes[i] *= from) )
      || ( !useFromURI && (m_Routes[i] *= to) ) )
    {
      if( !m_Routes[i].ReferenceRoute( connection ) )
      {
        B2BUACall * call = connection.GetLeg1Call();
        if( call )
          call->SetCallAnswerResponse( CallSession::DisconnectWithServiceNotAvailable );
        return FALSE;
      }

      if( m_Routes[i].HasPrimaryCodec() )
      {
        if( !(primaryCodec *= m_Routes[i].GetPrimaryCodec()) )
          continue;
      }

      const OStringArray & routes = m_Routes[i].GetTargetURIList();
      ok = routes.GetSize() != 0;

      if( ok )
      {
        for( PINDEX j = 0; j < routes.GetSize(); j++ )
        {
          SIPURI routeURI( routes[j] );
          
          if( routeURI.GetScheme() *= "enum" )
          {
            PStringArray enumServer;
            enumServer.AppendString( routeURI.GetHost().c_str() );
            SIPTransport::ENUMLookup(
              enumServer,
              targetName,
              routeURI ); 
            

          }

          if( routeURI.GetUser().IsEmpty() )
          {
            routeURI.SetUser( targetName );
          } else
          {
            if( !m_MaskTargetURI && 
              routeURI.GetUser().Find("*") != P_MAX_INDEX )
            {
              OString tempUser = routeURI.GetUser();
              tempUser.Replace( "*", targetName );
              routeURI.SetUser( tempUser );
            }

            if( useFromURI )
            {
              if( !ParserTools::WildCardCompare( routeURI.GetUser().c_str(), targetName.c_str() ) )
                continue;
            }
          }

          PIPSocket::Address verifiedAddress;
          WORD verifiedPort = 0;
          if( SIPTransport::Resolve( routeURI, verifiedAddress, verifiedPort, SIPTransport::UDP, m_ICMPValidate ) )
          {
            if( transportManager.IsLocalAddressAndPort( routeURI ) )
            {
              /// check if there is a registration 
              SIPMessage registration;
              if( regDb.FindRegistration( routeURI, registration ) )
              {
                Contact contact;
                if( !registration.GetContactAt( contact, 0 ) )
                  return FALSE;
                
                ContactURI contactURI;
                contact.GetURI( contactURI );
                SIPURI targetURI = contactURI.GetURI();
                  
                /// check the via if this is a NATted registration
                Via via;
                if( registration.GetViaAt( registration.GetViaSize() - 1, via ) 
                  && via.IsBehindNAT()  
                )
                {
                  targetURI.SetHost( via.GetReceiveAddress().AsString() );
                  targetURI.SetPort( OString( via.GetRPort() ) );
                }
                
                if( registration.IsEncrypted() )
                  targetURI.AddParameter( "ENCRYPTED", "TRUE" );

                connection.AddRoute( targetURI );
              }
            }else
            {
              connection.AddRoute( routeURI );
            }
          }
        }
      }
      break;
    }
  }

  return ok;

}

BOOL Router::HasRoute( 
  SIPURI & to 
)
{
  PWaitAndSignal lock( m_RoutesMutex );


  BOOL ok = FALSE;
  for( PINDEX i = 0; i < m_Routes.GetSize(); i++ )
  {
    if(  m_Routes[i] *= to )
    {
      ok = TRUE;
      break;
    }
  }

  return ok;
}

BOOL Router::FindRoute( 
  const SIPURI & uri,
  SIPURI & target
)
{
  PWaitAndSignal lock( m_RoutesMutex );

  BOOL ok = FALSE;
  for( PINDEX i = 0; i < m_Routes.GetSize(); i++ )
  {
    if(  m_Routes[i] *= uri )
    {
      const OStringArray & routes = m_Routes[i].GetTargetURIList();
      if( routes.GetSize() != 0 )
      {
        ok = TRUE;
        target = routes[0];
      }
      break;
    }
  }

  return ok;
}


BOOL Router::HasScheme(
  const SIPMessage & request,
  const OString & scheme
)
{
  PWaitAndSignal lock( m_RoutesMutex );

  if( !request.IsRequest() )
    return FALSE;

  /// get the To URI
  SIPURI to = request.GetToURI();

  OString userName = to.GetUser();
  
  for( PINDEX index = 0; index < m_Routes.GetSize(); ++index )
  {
    if( m_Routes[index] *= to )
      return m_Routes[index].HasScheme( scheme );
  }

  return FALSE;
}
