/*
 *
 * SBCAuthHandler.cxx
 * 
  * Open SIP Stack ( OSS )
 *
 * Copyright (c) opensipstack.org.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is OpenSIPStack.
 *
 * The Initial Developer of the Original Code is opensipstack.org.
 *
 * The author of this code is Joegen E. Baclor
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * The OpenSIPStack Library includes some GPL/LGPL code libraries that MAY be 
 * enabled at compile time using the --enable-gpllibs configure switch.  If
 * enabled, the content of this file is published under the terms of GNU 
 * General Public License Version 2.  For a detailed list of the available
 * GPL code, see $(opensipstack)/gnu/README file.
 *
 * $Log: SBCAuthHandler.cxx,v $
 * Revision 1.10  2007/07/14 07:53:56  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.9  2007/07/07 15:21:17  joegenbaclor
 * Added iproute2 support for linux
 *
 * Revision 1.8  2007/04/24 02:51:25  joegenbaclor
 * Fixed privacy bug where domain/host is not encoded with the identity
 *
 * Revision 1.7  2007/02/19 08:49:34  joegenbaclor
 * Changes PAssertedIdentity from RouteURI to ContactURI
 *
 * Revision 1.6  2007/02/16 11:09:20  joegenbaclor
 * More work on privacy
 *
 * Revision 1.5  2007/02/15 02:51:44  joegenbaclor
 * Completed trusted domain and proxy authentication support
 *
 * Revision 1.4  2007/02/14 11:22:09  joegenbaclor
 * Added Trusted Domain section
 *
 * Revision 1.3  2007/01/17 00:08:55  joegenbaclor
 * Added SysLog
 *
 * Revision 1.2  2006/08/29 07:45:42  rcolobong
 * Remove warning messages in Windows
 *
 * Revision 1.1  2006/08/14 10:04:59  rcolobong
 * Convert B2BUA to SBC
 * Support MP logging
 *
 * Revision 1.1  2006/07/11 14:34:38  joegenbaclor
 * Initial upload
 *
 *
 */

#include "B2BUA.h"
#include "SBCAuthHandler.h"

#define new PNEW

using namespace B2BUA;

static const char configKeySection[] = "Local Domain Accounts";
static const char configKeyAccount[] = "Account";

SBCAuthHandler::SBCAuthHandler(  
  OpenSBC & b2bua 
) : B2BAuthInterface( b2bua )
{
}

SBCAuthHandler::~SBCAuthHandler()
{
}

SBCAuthHandler::B2BAuthResponse SBCAuthHandler::B2BAuthenticate(
  B2BUAConnection & connection,
  SIPMessage & request,
  ProxyAuthenticate & proxyAuthenticate
)
{
  OpenSBC & sbc = dynamic_cast<OpenSBC&>(GetB2BUA());
  
  if( sbc.WillAcceptAllCalls() )
    return B2BAuthInterface::AuthAccept; /// just accept every call
  else
  {
    SIPParser::From from;
    if( request.GetFrom( from ) )
    {
      SIPURI uri = from.GetURI();
      if( sbc.IsTrustedDomain( uri ) )
        return B2BAuthInterface::AuthAccept;
    }
  }

  if( request.HasProxyAuthorization() )
  {
    ProxyAuthorization auth;
    if( !request.GetProxyAuthorization( auth ) )
    {
      PTRACE( 1, "No Proxy Authorization Header in request" );
      return B2BAuthInterface::AuthReject;
    }

    // WWW-Authenticate: Digest realm=voip.homeunix.org, nonce="48818999e02f8258f3c15c88f7c904e7", opaque="8d99e89591b57e262601a894391e4fe8", algorithm=MD5
    // Authorization: Digest username="2222", realm="voip.homeunix.org", algorithm=MD5, uri="sip:voip.homeunix.org", nonce="48818999e02f8258f3c15c88f7c904e7", opaque="8d99e89591b57e262601a894391e4fe8", response="b06feaf34eea2bd75533bea986e5fab8"
    OString userName;
    OString realm;
    OString uri;
    OString nonce;
    OString opaque;
    OString response;

    if( !auth.GetParameter( "username", userName ) )
    {
      PTRACE( 1, "No User Name specified in AUTHORIZATION" );
      return B2BAuthInterface::AuthReject;
    }

    if( !auth.GetParameter( "realm", realm ) )
    {
      PTRACE( 1, "No Realm specified in AUTHORIZATION" );
      return B2BAuthInterface::AuthReject;
    }

    if( !auth.GetParameter( "uri", uri ) )
    {
      PTRACE( 1, "No URI specified in AUTHORIZATION" );
      return B2BAuthInterface::AuthReject;;
    }

    if( !auth.GetParameter( "nonce", nonce ) )
    {
      PTRACE( 1, "No NONCE specified in AUTHORIZATION" );
      return B2BAuthInterface::AuthReject;
    }

    if( !auth.GetParameter( "response", response ) )
    {
      PTRACE( 1, "No Authorization response in AUTHORIZATION" );
      return B2BAuthInterface::AuthReject;
    }

    OString a1;
    OStringStream userURI;
    userURI << "sip:" << ParserTools::UnQuote( userName ) << "@" << ParserTools::UnQuote( realm );

    SIPURI account;
    SIPURI uuri(userURI.str());
    if( !sbc.FindLocalDomainAccount( uuri, account ) )
      return B2BAuthInterface::AuthReject;

    a1 = account.AsA1Hash();

    MD5::A2Hash a2( request.GetMethod(), uri );
    MD5::MD5Authorization localAuth = MD5::MD5Authorization( a1, ParserTools::UnQuote(nonce), a2 );
    
    OString md5Local =  localAuth.AsString();///MD5::MD5Authorization::Construct( a1.AsString(), ParserTools::UnQuote( nonce ), a2.AsString() );
    
    OString md5Remote = ParserTools::UnQuote( response ); 


    if( md5Local != md5Remote )
    {
      PTRACE( 1,  "Authorization token did not match: " 
        << " Local [" << md5Local << "]" << " Remote [" << md5Remote );

      LOG_IF_DEBUG( LogWarning(), "Authorization token did not match: " 
        << " Local [" << md5Local << "]" << " Remote [" << md5Remote );

      return B2BAuthInterface::AuthReject;
    }
    
    OnPostB2BAuthentication(
      connection,
      request,
      account );


    return B2BAuthInterface::AuthAccept;
  }


  
  /// we didn't find an authorization and this aint a trusted call 

  MD5::Nonce opaque;
  MD5::Nonce nonce;

  SIPParser::From from;
  OString host;
  if( request.GetFrom( from ) )
  {
    SIPURI uri = from.GetURI();
    host = uri.GetHost();
  }

  if( host.ToLower().Trim() == "anonymous.invalid" && !sbc.GetPrivacyDefaultRealm().IsEmpty() )
    host = sbc.GetPrivacyDefaultRealm();
    
  proxyAuthenticate.SetLeadString( "Digest" );
  proxyAuthenticate.AddParameter( "realm", host );
  proxyAuthenticate.AddParameter( "nonce", nonce.AsQuotedString() );
  proxyAuthenticate.AddParameter( "opaque", opaque.AsQuotedString() );
  proxyAuthenticate.AddParameter( "algorithm", "MD5" );

  return B2BAuthInterface::AuthChallenge;
}

BOOL SBCAuthHandler::RequireAuthorizationHash(
  const SIPURI & userURL,
  OString & a1,
  RegisterSession & 
)
{
  OpenSBC & sbc = dynamic_cast<OpenSBC&>(GetB2BUA());
  /// dont process the auth if sbc wants to accept all regs
  if( sbc.WillAcceptAllRegistration() )
    return TRUE;

  SIPURI account;
  if( !sbc.FindLocalDomainAccount( userURL, account ) )
    return FALSE;

  a1 = account.AsA1Hash();
  return TRUE;

}



void SBCAuthHandler::OnPostB2BAuthentication(
  B2BUAConnection & /*connection*/,
  SIPMessage & request,
  const SIPURI & uri
)
{
  OpenSBC & sbc = dynamic_cast<OpenSBC&>(GetB2BUA());

  /// check privacy
  if( request.HasPrivacy() )
  {
    Privacy privacy;
    request.GetPrivacy( privacy );

    if( privacy.GetHeaderBody().Trim() *= "id" )
    {
      SIPURI domainEntry;
      BOOL isTrusted = sbc.IsPrivacyTrustedDomain( uri, domainEntry );
      PAssertedIdentity identity;

      /// now we process trusted
      if( isTrusted )
      {
        ///check if we have a preferred identity
        if( request.GetPPreferredIdentitySize() > 0 )
        {
          PPreferredIdentity preferred = request.GetPPreferredIdentityAt(0);
          identity.SetHeaderBody( preferred.GetHeaderBody() );
          request.RemoveAllPAssertedIdentities();
          request.RemoveAllPPreferredIdentities();
          request.AppendPAssertedIdentity( identity );
        }else if( request.GetPAssertedIdentitySize() > 0 )
        {
          PAssertedIdentity preferred = request.GetPAssertedIdentityAt(0);
          request.RemoveAllPAssertedIdentities();
          request.RemoveAllPPreferredIdentities();
          request.AppendPAssertedIdentity( preferred );
        }else /// didn't provide  anything so lets use the account identity
        {
          OString accountIdentity;
          if( !uri.GetParameter( "identity", accountIdentity ) )
            accountIdentity = uri.GetUser() + "@" + uri.GetHost();
          else
            accountIdentity = accountIdentity + "@" + uri.GetHost();

          SIPURI identityURI(  accountIdentity );
          ContactURI curi;
          curi.SetURI( identityURI );
          identity.AddURI( curi );

          request.RemoveAllPAssertedIdentities();
          request.RemoveAllPPreferredIdentities();
          request.AppendPAssertedIdentity( identity );
        }
      }else
      {
          OString accountIdentity;
          if( !uri.GetParameter( "identity", accountIdentity ) )
            accountIdentity = uri.GetUser() + "@" + uri.GetHost();
          else
            accountIdentity = accountIdentity + "@" + uri.GetHost();

          SIPURI identityURI(  accountIdentity );
          ContactURI curi;
          curi.SetURI( identityURI );
          identity.AddURI( curi );

          request.RemoveAllPAssertedIdentities();
          request.RemoveAllPPreferredIdentities();
          request.AppendPAssertedIdentity( identity );
      }
    }
  }
}





