/*
 *
 * SBCCallHandler.cxx
 *
 * Open SIP Stack ( OSS )
 *
 * Copyright (c) opensipstack.org.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is OpenSIPStack.
 *
 * The Initial Developer of the Original Code is opensipstack.org.
 *
 * The author of this code is Joegen E. Baclor
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * The OpenSIPStack Library includes some GPL/LGPL code libraries that MAY be 
 * enabled at compile time using the --enable-gpllibs configure switch.  If
 * enabled, the content of this file is published under the terms of GNU 
 * General Public License Version 2.  For a detailed list of the available
 * GPL code, see $(opensipstack)/gnu/README file.
 *
 * $Log: SBCCallHandler.cxx,v $
 * Revision 1.9  2007/07/14 09:48:59  joegenbaclor
 * Corrected compile errors in linux
 *
 * Revision 1.8  2007/07/14 07:53:56  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.7  2007/07/07 15:21:17  joegenbaclor
 * Added iproute2 support for linux
 *
 * Revision 1.6  2007/07/06 14:18:25  rcolobong
 * Add Upstream Proxy Route feature
 *
 * Revision 1.5  2007/02/16 11:09:20  joegenbaclor
 * More work on privacy
 *
 * Revision 1.4  2007/02/09 03:50:30  joegenbaclor
 * 1. Added MaxForwards handling for B2B calls
 * 2. #defined SIP_DEFAULT_MAX_FORWARDS constant
 *
 * Revision 1.3  2007/02/08 13:38:12  joegenbaclor
 * Changed signature of B2BCallInterface::OnIncomingCall to allow for intial rejection
 *  of call before routing is performed.
 *
 * Revision 1.2  2006/08/29 07:45:42  rcolobong
 * Remove warning messages in Windows
 *
 * Revision 1.1  2006/08/14 10:04:59  rcolobong
 * Convert B2BUA to SBC
 * Support MP logging
 *
 * Revision 1.2  2006/06/29 05:09:12  joegenbaclor
 * Changed OnOutgoingCall invite parameter from const to none const to give a
 * chance for applications to modify the outbound invite before being sent to the transport
 *
 * Revision 1.1  2006/06/20 09:58:11  joegenbaclor
 * *** empty log message ***
 *
 *
 */


#include "SBCCallHandler.h"
#include "SBCRoutingHandler.h"
#include "B2BUAConnection.h"

#define DEFAULT_UPSTREAM_BACKDOOR_PORT 13900

static const char configKeyUpstreamProxySection[] = "Upstream Proxy";
static const char configKeyUpstreamSendToBackdoor[] = "Send all calls to backdoor port";

SBCCallHandler::SBCCallHandler( 
  OpenSBC & ua 
  ) : B2BCallInterface( ua )
{
}

SBCCallHandler::~SBCCallHandler()
{
}

CallSession::AnswerCallDeniedResponse SBCCallHandler::OnIncomingCall(
  B2BUAConnection & connection,
  B2BUACall & call,
  SIPMessage & invite
)
{
  return B2BCallInterface::OnIncomingCall( connection, call, invite );
}

void SBCCallHandler::OnOutgoingCall(
  B2BUAConnection & connection,
  B2BUACall & call,
  SIPMessage & invite
)
{
  OpenSBC & sbc = dynamic_cast<OpenSBC&>(GetB2BUA());

  /// check privacy
  if( invite.HasPrivacy() )
  {
    Privacy privacy;
    invite.GetPrivacy( privacy );

    if( privacy.GetHeaderBody().Trim() *= "id" )
    {
      SIPURI uri;
      if( invite.GetRequestURI( uri )  )
      {
        SIPURI domainEntry;
        if( !sbc.IsPrivacyTrustedDomain( uri, domainEntry ) )
        {
          invite.RemoveAllPAssertedIdentities();
          invite.RemoveAllPPreferredIdentities();
        }
      }
    }
  }

  // Remove invite upstream route
  OpenSBC::UpstreamProxyRoute inviteUpstreamRoute;
  SIPHeader customHeader( inviteUpstreamRoute.GetHeaderName() );
  while( invite.PopCustomHeader( 
    inviteUpstreamRoute.GetHeaderName().c_str(), customHeader ) ) 
  {
    LOG_CONTEXT( LogInfo(), call.GetCallId(), "Upstream Proxy Route: Pop Header " 
      << customHeader );
  }

  inviteUpstreamRoute = customHeader.AsString();

  // get current upstream proxy route and send it to
  // the invite send address
  if( !customHeader.GetHeaderBody().IsEmpty() )
  {
    SIPURI upstreamRoute = inviteUpstreamRoute.GetHeaderBody();
    LOG_CONTEXT( LogInfo(), call.GetCallId(), "Upstream Proxy Route: Set Invite Send Address to " 
      << upstreamRoute );
    invite.SetSendAddress( upstreamRoute );
  } else
  {
    OSSAppConfig * config = OSSAppConfig::GetInstance();
    /// lock the config to make sure noone can change data via http admin until we are done
    PWaitAndSignal configLock( PHTTPConfig::m_HTTPConfigMutex );

    BOOL sendToBackdoorURI = config->GetBoolean( 
      configKeyUpstreamProxySection, 
      configKeyUpstreamSendToBackdoor, FALSE );

    if( sendToBackdoorURI )
    {
      LOG_CONTEXT( LogInfo(), call.GetCallId(), "Upstream Proxy Route: " 
        << "ALWAYS SEND TO THE BACKDOOR PORT " );

      B2BUACall * leg1Call = connection.GetLeg1Call();
      if( leg1Call == NULL )
        return;

      B2BUACall::GCRef ref = GCCREATEREF( leg1Call, "SBCCallHandler::OnOutgoingCall",
        "B2BUACall Leg 1" );
      if( ref == NULL )
        return;

      Via via;
      if( leg1Call->GetCurrentUASInvite().GetViaAt( 0, via ) )
      {
        SIPURI upstreamRoute = via.GetURI();
        upstreamRoute.SetPort( OString( DEFAULT_UPSTREAM_BACKDOOR_PORT ) );

        LOG_CONTEXT( LogInfo(), call.GetCallId(), "Upstream Proxy Route: Set Invite Send Address to " 
          << upstreamRoute );

        invite.SetSendAddress( upstreamRoute );
      }
    }
  }

  // if this call is upstream proxy
  // add custom heared upstream proxy route
  if( connection.IsUpstreamProxy() )
  {
    OString backdoorURI =
      connection.GetUpstreamBackdoorURI();
    if( backdoorURI.IsEmpty() )
    {
      LOG_IF_DEBUG( LogError(), "Upstream Proxy: Backdoor URI is Empty!!!" );
      return;
    }

    SBCRoutingHandler::B2BRouteList routeList =
      connection.GetUpstreamProxyRoutes();
    if( routeList.IsEmpty() )
    {
      LOG_IF_DEBUG( LogError(), "Upstream Proxy: Route is Empty!!!" );
      return;
    }

    PIPSocket::Address targetAddress;
    WORD targetPort;
    SIPURI uri = routeList[0];
    if( !SIPTransport::Resolve( uri, targetAddress, targetPort ) )
    {
      LOG_IF_DEBUG( LogWarning(), "Upstream Proxy: does not resolve to an IP Address: " 
        << uri );
      return;
    }

    OpenSBC::UpstreamProxyRoute upstreamHeader;
    upstreamHeader.AddURI( SIPURI( backdoorURI ) );
    upstreamHeader.PrepareBody();
    
    invite.AddCustomHeader( upstreamHeader );
    invite.SetSendAddress( uri );
  }
}

void SBCCallHandler::OnCallReinvite(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnProceeding(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnCallAbandoned(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnAlerting(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnProgress(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnRejected(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnConnected(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnDisconnected(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

void SBCCallHandler::OnInSessionMessage(
  B2BUAConnection &,
  B2BUACall &,
  const SIPMessage &
)
{
}

